/* simulate sample paths geometric brownian motion */
/*
	Copyright 2001 Gunter Winkler
*/

#ifndef __INC__gbm__
#define __INC__gbm__ 1

#include "fastrand.h"

double wiener_final(PMTDATA handle, const double mu, const double sigma,
		const double start, const double dt, const int steps);
/* 
	simulate 'steps' steps of width 'dt' of a Wiener process, starting 
	at S(0)='start', with drift 'mu', variance 'sigma'
	result is S(steps*dt)
*/

double gbm_final(PMTDATA handle, const double mu, const double sigma,
		const double start, const double dt, const int steps);
/* 
	simulate 'steps' steps of width 'dt' of a gbm, starting with S(0)='start',
	drift 'mu', variance 'sigma'
	result is S(steps*dt)
	(gbm_final calls wiener_final with mu = mu - sigma^2/2 and start=0)
*/

double gbm2_final(PMTDATA handle, const double mu, const double sigma,
		const double start, const double dt, const int steps);
/* 
	simulate 'steps' steps of width 'dt' of a gbm, starting with S(0)='start',
	drift 'mu', variance 'sigma'
	result is S(steps*dt)
	(gbm2_final uses dS = S (mu dt + sigma dW) )
*/

int gbm_barrier(PMTDATA handle, const double mu, const double sigma,
		double & S, const double dt, const int steps, 
		const double barrier);
/* 
	simulate 'steps' steps of width 'dt' of a gbm, starting with given S(0),
	drift 'mu', variance 'sigma'
	The simulation stops if the spot hits the barrier ( (spot-barrier) changes 
	the sign ). The return value is	-1 if the barrier was not hit and the 
	step number otherwise.
	see also: gbm2_final
*/
#endif
