/* simulate sample paths of hestons motion */
/*
	Copyright 2001 Gunter Winkler
*/

#include "heston.h"
#include <cassert>
#include <iostream>

using std::cout;
using std::endl;

static inline int sign(const double x)
{
	if (x>0.0) return 1;
	if (x<0.0) return -1;
	return 0;
}

void heston_final(PMTDATA handle, 
		double & spot, double & variance,
		const MARKETDATA market,
		const double dt, const int steps)
/* 
	simulates Heston's stochastic volatility model and returns final
	values of spot and variance.
*/
{
	int i;
	double x1,x2;
	double sdt = sqrt(dt);
	double mu = market.rd - market.rf;
	
	for (i=steps; i>0; --i)
	{
		gencorrnormal(handle,x1,x2,market.rho);
		spot += spot * (mu * dt + sqrt(variance) * x1 * sdt);
		variance += market.kappa *(market.theta-variance) * dt 
				+ market.xi * sqrt(variance) * x2 * sdt;
//		cout << i << ", " << spot << ", " << variance << endl;
		if (variance < min_variance) variance = min_variance;  
		/* FIXME: bad style ... find better process */
		assert( variance > 0.0 );
	}
}

int heston_barrier(PMTDATA handle, 
		double & spot, double & variance,
		const MARKETDATA market,
		const double dt, const int steps,
		const double barrier)
/* 
	simulates Heston's stochastic volatility model and returns final
	values of spot and variance. The simulation stops if the spot hits
	the barrier ( (spot-barrier) changes the sign ). The return value is
	-1 if the barrier was not hit and the step number otherwise.
*/
{
	int i;
	double x1,x2;
	double sdt = sqrt(dt);
	double mu = market.rd - market.rf;
	int updown;
	
	updown = sign(spot-barrier);
	if (updown==0) return 0;
	/* abort if the option starts at the barrier */
	
	for (i=steps; i>0; --i)
	{
		gencorrnormal(handle,x1,x2,market.rho);
		spot += spot * (mu * dt + sqrt(variance) * x1 * sdt);
		variance += market.kappa *(market.theta-variance) * dt 
				+ market.xi * sqrt(variance) * x2 * sdt;
//		cout << i << ", " << spot << ", " << variance << endl;
		if (variance < min_variance) variance = min_variance;  
		/* FIXME: bad style ... find better process */
		assert( variance > 0.0 );
		if (sign(spot-barrier) != updown) return (steps-i);
	}
	return -1;
}
