/* simulate sample paths of hestons motion */
/*
	Copyright 2001 Gunter Winkler
*/

#ifndef __INC__heston__
#define __INC__heston__ 1

#include <cmath>
#include "fastrand.h"
#include "otypes.h"

#define min_variance	1.0e-10
/* minimal variance, too small values can cause numerical instability */

void heston_final(PMTDATA handle, 
		double & spot, double & variance,
		const MARKETDATA market,
		const double dt, const int steps);
/* 
	simulates Heston's stochastic volatility model and returns final
	values of spot and variance.
	parameters:
		handle ... handle of pseudo-random number generator data
		spot ..... IN: current spot rate, OUT: terminal spot rate
		variance . IN: initial variance, OUT: terminal variance
		market ... market and model parameters
		dt ....... step size
		steps .... number of steps, thus simulation period T = dt * steps
		           steps should be a even number.
*/

int heston_barrier(PMTDATA handle, 
		double & spot, double & variance,
		const MARKETDATA market,
		const double dt, const int steps,
		const double barrier);
/* 
	simulates Heston's stochastic volatility model and returns final
	values of spot and variance. The simulation stops if the spot hits
	the barrier ( (spot-barrier) changes the sign ). The return value is
	-1 if the barrier was not hit and the step number otherwise.
	see also: heston_final
*/

#endif
