/*
   a short description from the paper mt.ps of Makoto Matsumoto and 
   Takuji Nishimura
   see: http://www.math.keio.ac.jp/matumoto/emt.html
   
In this paper, a new algorithm named Mersenne Twister (MT) for generating 
uniform pseudoran- dom numbers is proposed. For a particular choice of 
parameters, the algorithm provides a super astronomical period of 2^19937-1

We implemented this generator as a portable C-code. It passed several stringent 
statistical tests, including diehard. The speed is comparable to other modern 
generators.
*/

#ifndef __INC__mt19937__
#define __INC__mt19937__ 1

/* copied from mt19937.c; DON'T change this */
#define __mt__N 624
struct MTDATA 
{
	int mti;           /* mti==N+1 means mt[N] is not initialized */
	unsigned long mt[__mt__N]; /* the array for the state vector  */
};

typedef MTDATA* PMTDATA;

void sgenrand(PMTDATA& handle, unsigned long seed = 4357);
/* initialize the generator with an nonzero integer */

void freegenrand(PMTDATA& handle);
/* free all memory allocated for this handle */

double genrand(PMTDATA handle);
/* return a pseudo-random number from the interval [0,1] */

#endif
