/*
	Copyright 2001 Gunter Winkler
*/

#include <iostream>
#include "fastrand.h"
#include "gbm.h"
#include "heston.h"

using std::cout;
using std::endl;

static inline double sqr(double a) { return a*a; }

int main(int argc, char* argv[])
{
	long COUNT = 5000;
	double gbms[5000];
	
	double dt = 0.0025;
	int steps = 1000;
	
	long i,anz;
	double x1,x2;
	double rho = 0.1, sqrtrho=sqrt(1-rho*rho);
	double sum1, sum2, sqr1, sqr2, cor;
	double strike,profit,sigma,barrier;
	
	int hit;
	
	double spot,variance;
	MARKETDATA market;
		
	PMTDATA handle;
	
	sgennormal(handle,4245);
// 	cout.setf(std::ios::fixed, std::ios::floatfield);
 	cout.precision(6);
// 	cout.width(10);	
	sum1 = 0.0; 
	sum2 = 0.0; 
	sqr1 = 0.0;
	sqr2 = 0.0;
	cor  = 0.0;
	for (i=0; i<COUNT; ++i)
	{
		gencorrnormal(handle,x1,x2,rho,sqrtrho);
		// cout << x1 << x2;
		sum1 += x1;
		sum2 += x2;
		sqr1 += x1*x1;
		sqr2 += x2*x2;
		cor  += x1*x2;
	}
	cout << endl;
	cout << "E x1 = " << (sum1/COUNT) << endl;
	cout << "E x2 = " << (sum2/COUNT) << endl;
	cout << "E x1^2 = " << (sqr1/COUNT) << endl;
	cout << "E x2^2 = " << (sqr2/COUNT) << endl;
	cout << "E x1x2 = " << (cor /COUNT) << endl;
	cout << "D x1 = " << (sqr1/COUNT - sqr(sum1/COUNT)) << endl;
	cout << "D x2 = " << (sqr2/COUNT - sqr(sum2/COUNT)) << endl;
	cout << "D x1x2 = " << (cor /COUNT - (sum1/COUNT)*(sum2/COUNT)) << endl;
	cout << endl;
	
	/**********************/
	
	sigma = sqrt(0.06);
	cout << "gbm1 = " << gbm2_final(handle,0.02,sigma,1.0,dt,steps) << endl;
	sum1 = 0.0; 
	sum2 = 0.0; 
	sqr1 = 0.0;
	sqr2 = 0.0;
	cor  = 0.0;
//	COUNT /= 10;
	
	strike = 1.0;
	profit = 0.0;

	x1 = gbm2_final(handle,log(1.04),sigma,1.0,dt,steps);
	gbms[0]=x1;
	sum2 = sum1;
	sum1 += x1;
	sqr1 += 0.0;
	if (x1>strike) profit += x1-strike;
	for (i=1; i<COUNT; ++i)
	{
		x1 = gbm2_final(handle,log(1.04),sigma,1.0,dt,steps);
		gbms[i]=x1;
		sum2 = sum1;
		sum1 += x1;
		if (x1>strike) profit += x1-strike;
	}
	x1 = (sum1/COUNT);
	for (i=0; i<COUNT; ++i)
	{
		sqr2 += sqr(gbms[i]-x1);
	}
	cout << "E x1 = " << (sum1/COUNT) << endl;
	cout << "D x1 = " << (sqr2/(COUNT-1)) << endl;
	cout << "E (S-K)+ = " << (profit/COUNT) << endl;
	cout << endl;
	
	/**********************/
	
	market.rd     = log(1.04); 
	market.rf     = log(1.00); 	/* (logarithmic) domestic/foreign interest rates */
	market.rho    = -0.2; 
	market.kappa  = 2.5; 
	market.theta  = 0.06; 
	market.xi     = 0.5;	/* stochastic volatility parameters */
	market.lambda = 0.0; 	/* price of volatility risk */
	
	spot = 1.0;
	variance = market.theta;
	
	sum1 = 0.0; 
	sum2 = 0.0; 
	sqr1 = 0.0;
	sqr2 = 0.0;
	cor  = 0.0;
	profit = 0.0;
	
	heston_final(handle, spot, variance, market, dt, steps);
	cout << "heston: spot " << spot << " var. " << variance << endl;
	
	for (i=0; i<COUNT; ++i)
	{
		spot = 1.0;
		variance = market.theta;
		heston_final(handle, spot, variance, market, dt, steps);
		gbms[i]=spot;
		sum1 += spot;
		sum2 += variance;
		if (spot>strike) profit += spot-strike;
	}
	x1 = (sum1/COUNT);
	for (i=0; i<COUNT; ++i)
	{
		sqr2 += sqr(gbms[i]-x1);
	}
	cout << "E x = " << (sum1/COUNT) << endl;
	cout << "E v = " << (sum2/COUNT) << endl;
	cout << "D x = " << (sqr2/(COUNT-1)) << endl;
	cout << "E (S-K)+ = " << (profit/COUNT) << endl;
	cout << endl;

	/**********************/

	spot = 1.0;
	variance = market.theta;
	barrier = 0.95;
	
	sum1 = 0.0; 
	sum2 = 0.0; 
	sqr1 = 0.0;
	sqr2 = 0.0;
	cor  = 0.0;
	profit = 0.0;
	anz = 0;
	
	hit = gbm_barrier(handle,market.rd-market.rf,sigma,spot,dt,steps,barrier);
	cout << "gbm-barrier: spot " << spot << " var. " << variance 
			<< " hit " << hit << endl;

	for (i=0; i<COUNT; ++i)
	{
		spot = 1.0;
		variance = market.theta;
		hit = gbm_barrier(handle,market.rd-market.rf,sigma,spot,dt,steps,barrier);
		if ((hit==-1) && (spot>strike)) profit += spot-strike;
		if (hit>0) ++anz;
	}
	cout << " barrier: " << barrier << endl;
	cout << " knock-outs: " << 100.0*(double)anz/COUNT << "%" << endl;
	cout << "E (S-K)+ = " << (profit/COUNT) << endl;

	/**********************/

	spot = 1.0;
	variance = market.theta;
//	barrier = 1.05;
	
	sum1 = 0.0; 
	sum2 = 0.0; 
	sqr1 = 0.0;
	sqr2 = 0.0;
	cor  = 0.0;
	profit = 0.0;
	anz = 0;
	
	hit = heston_barrier(handle, spot, variance, market, dt, steps, barrier);
	cout << "heston-barrier: spot " << spot << " var. " << variance 
			<< " hit " << hit << endl;

	for (i=0; i<COUNT; ++i)
	{
		spot = 1.0;
		variance = market.theta;
		hit = heston_barrier(handle, spot, variance, market, dt, steps, barrier);
		if ((hit==-1) && (spot>strike)) profit += spot-strike;
		if (hit>0) ++anz;
	}
	cout << " barrier: " << barrier << endl;
	cout << " knock-outs: " << 100.0*(double)anz/COUNT << "%" << endl;
	cout << "E (S-K)+ = " << (profit/COUNT) << endl;

	freegennormal(handle);
	
	return 0;
}
